import fetch from "node-fetch";
const userInfoByToken = {};
export const getUserInfo = async (accessToken) => {
    const prevPromise = userInfoByToken[accessToken];
    if (prevPromise) {
        return prevPromise;
    }
    const promise = (async () => {
        const response = await fetch("https://api.github.com/user", {
            headers: { Authorization: `token ${accessToken}`, Accept: "application/vnd.github+json" },
        });
        const jsonResponse = (await response.json());
        if (!jsonResponse || !jsonResponse.name || !jsonResponse.email) {
            throw new Error("Failed to get user info");
        }
        const result = { name: jsonResponse.name, email: jsonResponse.email };
        return result;
    })();
    userInfoByToken[accessToken] = promise;
    return promise;
};
const reposByName = {};
export const getUserRepos = async (accessToken) => {
    const userInfo = await getUserInfo(accessToken);
    const prevPromise = reposByName[userInfo.name];
    if (prevPromise) {
        return prevPromise;
    }
    const promise = (async () => {
        const response = await fetch("https://api.github.com/user/repos?sort=created&per_page=100&affiliation=owner", {
            headers: { Authorization: `token ${accessToken}`, Accept: "application/json" },
        });
        const jsonResponse = (await response.json());
        if (!jsonResponse || !Array.isArray(jsonResponse)) {
            throw new Error("Failed to get user repos");
        }
        const checkedRepos = jsonResponse?.flatMap(({ name, full_name, description, owner, visibility, archived }) => {
            if (name && full_name && owner && owner.login && visibility && archived != null) {
                return [
                    {
                        name: name,
                        fullName: full_name,
                        ...(description ? { description } : {}),
                        owner: owner.login,
                        visibility: visibility,
                        archived: archived,
                    },
                ];
            }
            return [];
        });
        if (checkedRepos.length !== jsonResponse.length) {
            throw new Error("Got invalid repos");
        }
        return checkedRepos;
    })();
    reposByName[userInfo.name] = promise;
    return promise;
};
export const createGithubRepo = async (accessToken, name, description) => {
    const userInfo = await getUserInfo(accessToken);
    const body = {
        name: name,
        description: description,
        homepage: `https://github.com/${userInfo.name}/${name}`,
        private: false,
        has_projects: false,
        has_wiki: false,
        auto_init: true,
        license_template: "MIT",
        has_downloads: false,
    };
    const response = await fetch("https://api.github.com/user/repos", {
        headers: { Authorization: `token ${accessToken}`, Accept: "application/vnd.github+json" },
        method: "POST",
        body: JSON.stringify(body),
    });
    if (response.status !== 201) {
        throw new Error("Failed to create repo");
    }
    delete reposByName[userInfo.name];
    await getUserRepos(accessToken);
};
export const getDefaultBranch = async (accessToken, repoName) => {
    const userInfo = await getUserInfo(accessToken);
    const response = await fetch(`https://api.github.com/repos/${userInfo.name}/${repoName}`, {
        headers: { Authorization: `token ${accessToken}`, Accept: "application/vnd.github+json" },
    });
    const jsonResponse = (await response.json().catch(() => undefined));
    if (!jsonResponse || !jsonResponse.default_branch) {
        return undefined;
    }
    return jsonResponse.default_branch;
};
